function [stc, P_C, A_C] = GSC_CCF(rho,lam,A_,As,S_k,Scale,L)
%__________________________________________________________________________
% PROCEDURE GSC_CCF      
%  Calculates CCF of GSCM (for linear combinations L)            
% - INPUT  rho    : Cycle     decay
%          lam    : Frequency decay
%          A, A_st: as in proposition 1                              (nxn)
%                   In practice the blocks of S.Z related to cycles 
%          S_k    : Covariance matrix of cyclical innovations        (nxn) 
%          Scale  : Scale factor for calculating StdDev Cycles
%                     this is needed for the scaled PD model
%                     if A_/As are taken from Z then Scale=eye(n)
%          L      : Matrix of linear combinations                    (kxn) 
% - OUTPUT        : None                                          
%__________________________________________________________________________
% Basic matrices
  T_      =  eye(2);
  Ts      =  [0 1;-1 0];
  P       =  [A_ As];
 
  B_      =  (L*P) * kron(T_,S_k) * (L*P)';
  Bs      =  (L*P) * kron(Ts,S_k) * (L*P)';
  B_      =  fix(B_*1e9) /1e9;                    % rounds of towards 0.
  Bs      =  fix(Bs*1e9) /1e9;
    
% Phase Ph_C and association A_C
  B_sign  =  abs(B_)./B_;
  P_C     =  atan(Bs./B_) / lam; 
  Co_C    =  B_.*B_ + Bs.*Bs;
  R_C     =  sqrt(Co_C) .* B_sign;
  A_C     =  cor_mat(R_C);
  
  stc     =  Scale * sqrt(diag(R_C)) ./ sqrt(1-rho^2);

% Output  
  disp(' ')
  disp(['' kron('_',ones(1,75))])
  disp('Similar Cycles ACGF')
  disp(' ') 
  disp('StdDev Cycles*100')
  disp(num2str(stc'*100,'%10.4f'))
  disp(' ')
  disp('Relative to cycle 1')
  disp(num2str(sqrt(stc'/stc(1)),'%10.4f'))
 
  disp(' '); 
  disp('Phase shifts')
  disp(num2str(P_C,'%10.4f'))
  disp(' '); 
  disp('Phase adjusted associations')
  disp(num2str(A_C,'%10.4f'))
     
% CCF 
  k      =  size(L,1);
  disp(' '); 
  disp('Cross correlation function');
  for lag = 0:12
       CCf  = zeros(k,k);
       for i = 1:k 
       for j = 1:k
         CCf(i,j) = (rho^lag)*A_C(i,j)*cos(lam*(lag-P_C(i,j)));
       end
       end
      disp(' ')
      disp(['Lag ' num2str(lag,'%5.0f')])
      disp(num2str(CCf,'%10.4f'))
   end
end
