%__________________________________________________________________________
% MAIN PROGRAM FOR THE DIFFUSE TIME VARIANT KALMAN-FILTER         
% Estimates the stacked univariate models for US GDP/CTR/HPR.
% Slopes for CTR and HPR restricted to .001. 
% Similar cycles restrictions on all 3 cycles.
%__________________________________________________________________________
  clear
  clc
  Base_Dir = cd; 
  addpath(genpath(Base_Dir));
   
%__________________________________________________________________________
% Data
%__________________________________________________________________________
  ctry     = 'US';    
  load([Base_Dir '\Data 2015-07-10 US'],'SERIES','Date','X') 
 
  Ym       = X(13:end,1:3);
  Date     = Date(13:end,:);
  Label{1} = [ctry,': GDP'];
  Label{2} = [ctry,': CTR'];
  Label{3} = [ctry,': HPR'];
  
%__________________________________________________________________________
% Model
% p(40) - p(42) contain the phase shifts xi(2) ... xi(n)
% p(43) - p(45) contain the elements of X where D =  (I-X)^(1) (I+X).
%               and X is skew-symmetric
% Given that the 1st row/col of D is unity, the same applies to X
%__________________________________________________________________________
  M_.Model   =  'GF3_S123_US';
  
  M_.Trend   =  1:3;
  M_.Slope   =  4:6;
  M_.Cycle   =  13:18;
   
% Parameter names 
% Cyc dynamics
  M_.p_n{1}  = 'rho1' ;    
  M_.p_n{4}  = 'lam1' ;
  M_.p_n{7}  = 'car1' ; 
    
% Trends and slope  
  M_.p_n{10} = 'RW1 ' ; M_.p_n{11} = 'RW2 ' ; M_.p_n{12} = 'RW3' ; 
  M_.p_n{14} = 'RW21' ; M_.p_n{15} = 'RW31' ; M_.p_n{16} = 'RW32'; 
 
  M_.p_n{20} = 'SL1 ' ; M_.p_n{21} = 'SL2 ' ; M_.p_n{22} = 'SL3 '; 
  M_.p_n{24} = 'SL21' ; M_.p_n{25} = 'SL31' ; M_.p_n{26} = 'SL32'; 
 
% Factor loadings  
  M_.p_n{30} = 'A11 ' ;
  M_.p_n{33} = 'A21 ' ; M_.p_n{34} = 'A22 ' ; 
  M_.p_n{36} = 'A31 ' ; M_.p_n{37} = 'A32 ' ; M_.p_n{38} = 'A33 ';

  M_.p_n{40} = 'A*21' ; 
  M_.p_n{43} = 'A*31' ; M_.p_n{44} = 'A*32' ; 
  

% Irreg  
  M_.p_n{50} = 'eps1' ; M_.p_n{51} = 'eps2' ; M_.p_n{52} = 'eps3' ;
  
  M_.p_n{55} = 'dum1' ; M_.p_n{56} = 'dum2' ; M_.p_n{57} = 'dum3';
  M_.p_n{58} = 'dum4' ; M_.p_n{59} = 'dum5' ; M_.p_n{60} = 'dum6';
 

%__________________________________________________________________________
%  STACKED UNIVARIATE 
%  RANDOM WALK
%  CTR AND HOUSE PRICE SLOPE RESTRICTED
%  Likhood  = 2262.26751773
%__________________________________________________________________________
  M_.p0     =  zeros(1,60);
  M_.px     =  zeros(1,60);
  M_.ps     =   ones(1,60);
  M_.p0(1)  =  0.277014086578;      M_.ps(1)  = 1;         M_.px(1)  = 1;      
  M_.p0(4)  =  0.141174746100;      M_.ps(4)  = 1;         M_.px(4)  = 1;      
  M_.p0(7)  =  3.520177988879;      M_.ps(7)  = 1;         M_.px(7)  = 1;      
  M_.p0(10) =  0.006491428743;      M_.ps(10) = 100;       M_.px(10) = 1;      
  M_.p0(11) =  0.002425872702;      M_.ps(11) = 1000;      M_.px(11) = 1;      
  M_.p0(12) =  0.000008173147;      M_.ps(12) = 1000;      M_.px(12) = 1;      
  M_.p0(20) =  0.000120837916;      M_.ps(20) = 1000;      M_.px(20) = 0;      
  M_.p0(21) =  0.001000000000;      M_.ps(21) = 1000;      M_.px(21) = 0;      
  M_.p0(22) =  0.001000000000;      M_.ps(22) = 1000;      M_.px(22) = 0;      
  M_.p0(30) =  0.002183825562;      M_.ps(30) = 100;       M_.px(30) = 1;      
  M_.p0(34) =  0.002578335378;      M_.ps(34) = 100;       M_.px(34) = 1;      
  M_.p0(38) =  0.007603967852;      M_.ps(38) = 100;       M_.px(38) = 1;      
  M_.p0(55) =  0.014655877373;      M_.ps(55) = 1;         M_.px(55) = 1;      
  M_.p0(56) = -0.018060614295;      M_.ps(56) = 1;         M_.px(56) = 1;      
  M_.p0(57) = -0.017509674293;      M_.ps(57) = 1;         M_.px(57) = 1;
  
  
%__________________________________________________________________________
% Estimation options
%__________________________________________________________________________
  opt_flag   = 0;                                      
  smo_flag   = 1;                                   
  tst_flag   = 1;
  
  options = optimset('fminunc');
  options = optimset(options,'Display'       ,'iter');  
  options = optimset(options,'FunValCheck'   ,'on');
  options = optimset(options,'TolFun'        ,1e-5);
  options = optimset(options,'TolX'          ,1e-5);
  options = optimset(options,'MaxFunEvals'   ,20000);
  options = optimset(options,'MaxIter'       ,100);
  
%__________________________________________________________________________
% Parameter restrictions
% Set M_.px(i)= 0 to exclude M_.p(i) from estimation, M_.px(i)= 1 otherwise
%
%  The Similar Cycles restriction for 3 similar cycles implies that
%  A  elements (1,2) = (1,3) = (2,3) = 0
%  A* elements (2,2) = (2,3) = (3,3) = 0
%  Sig_Kappa = 0 --> p(46:48) = 0
%__________________________________________________________________________
% Parameter restrictions
  M_.px(1)     = [1];                   % Rho    1-2
  M_.px(4)     = [1];                   % Lambda 1-2
  M_.px(7)     = [1];                   % CAR1   1-2

  M_.px(50:52) = [0 0 0];               % Irregular   
  
  M_.px(10:12) = [1 1 1];               % RW  
  M_.px(14:16) = [0 0 0];
  
  M_.px(20:22) = [1 0 0];               % Slope 
  M_.px(24:26) = [0 0 0];
  
  M_.px(30:38) = [1 0 0    ...          % A   (1,2) = (1,3) = (2,3) = 0
                  0 1 0    ...
                  0 0 1];                 

  M_.px(40:45) = [0 0 0 ... 
                  0 0 0];               % A*  (2,2) = (2,3) = (3,3) = 0
  
  M_.px(46:48) = [0 0 0];               % corr Sig_kappa 
                                        % element (1,3) = 0!!
  M_.px(55:59) = [1 1 1 0 0];           % Dummies 

  
%__________________________________________________________________________
%  Estimation                                                    
%__________________________________________________________________________
  p_  = select(M_.p0,M_);
   if opt_flag 
     [p_,lv,flag,o_,g_,h_] = fminunc('DKFLoglik',p_,options,Ym,M_);
      cov_  = inv(h_);
   else
      lv    = DKFLoglik(p_,Ym,M_);
      cov_  = nan;
      flag  = 9;
   end
   p_  = deselect(p_,M_.p0,M_);
   eval(['S = ' M_.Model '(p_,[],0);'])
   R   = DKF(Ym,M_.Model,p_);
   opt_outp(flag,lv,p_,cov_,M_,size(Ym,1)); 
   
%__________________________________________________________________________
% Smoothing                                                  
%__________________________________________________________________________
  if smo_flag
     R_s    = AKF(Ym ,M_.Model,p_); 
     R_s    = ADS(R_s,M_.Model,p_);
     R_s    = STS(R_s,M_.Model,p_);
     [Trends,Slopes,Cycles] = graphs_mv(Date,Ym,S.Z,R,R_s,M_,Label);
  end
  
%__________________________________________________________________________
% Diagnostics
%__________________________________________________________________________
  if tst_flag
      Sv_      = UOC_tests(Date,Ym,R,S.nd);        
      SG_      = GFC_Diagnostics(Ym,M_,S,p_,'',ctry);
  end
  
%__________________________________________________________________________
% Printing new starting values                                      
%__________________________________________________________________________ 
  Print_pars(p_,M_.ps,M_.px,lv,0)
